; 
; File Name		:'ISR.asm"
; Title			:8515 Interrupt Handlers
; Date			:
; Version		:
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Target MCU		:AT90S8515
;
; DESCRIPTION
; All interrupt handlers reside here
;
;***************************************************************************;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.07.29  dvh	Creation
;	0.02	98.08.20  dvh	Added Timer 1 int for servo control, a little cleanup
;				and better documentation elsewhere
;	0.03	98.08.21  dvh	Added support in TIM0_OVF for a FRAME_DELAY
;				variable used in SERVO.ASM. 
;				Converted Frame_Delay to SRAM from register
;	0.04	98.08.23  dvh	Regularized to XL,XH notation instead of R26.R27 etc
;	0.05	99.09.09  dvh	Fixed morse handler to work properly.
;
;***************************************************************************
;Fixed jump vectors
;
;$000
	rjmp	INIT_Machine	;Restart vector, points to the beginning of the code. 
;$001
	rjmp	EXT_INT0	;IRQ 0
;$002
	rjmp	EXT_INT1	;IRQ 1 
;$003	
	rjmp	TIM1_CAPT	;Timer 1 capture 
;$004
	rjmp	TIM1_COMPA	;Timer 1 compare A
;$005
	rjmp	TIM1_COMPB	;Timer 1 compare B
;$006
	rjmp	TIM1_OVF	;Timer 1 overflow
;$007	
	rjmp	TIM0_OVF	;Timer 0 overflow
;$008	
	rjmp	SPI_Handler	;Serial transfer complete
;$009
	rjmp	UART_RXC	;Uart RX complete
;$00A
	rjmp	UART_DRE	;Uart data register empty
;$00B
	rjmp	UART_TXC	;Uart TX complete
;$00C	
	rjmp	ANA_COMP	;Analog comparator
;
;**************************************************************
;
;All my ISRs use TTEMP and TTEMP2 to maximize speed. You could push
;TEMP, and pop it back.. 
;
;Each int vector has, at minimum, code that turns off that int. 
;This is a "belts and suspenders" method that protects me from
;any ints I forgot to turn off, or accidentally enabled.
;
;If you decide to use one of the disabled ints, enable it in the
;code when appropriate, and replace these "turn-off" handlers
;with whatever it is you want done.
;
;Each ISR saves SREG to TTEMP2, which is dedicated to this purpose.
;I could have pushed and popped it, but push and pop take two cycles,
;and in and out only take one. If you get squeezed for registers, then
;you can always do the pushpop dance. It's an extra 500nS at 8MHz
;
;MAKE ABSOLUTELY SURE YOU DO NOT USE "TEMP" IN ANY ISR! (unless you push and pop it!)
;
;
;**************************************************************
;
;Unused at this point. 
;
;
EXT_INT0:	
	in	TTEMP2,SREG	;Saves the status register

INT0_OFF:
	in	TTEMP,GIMSK	;If done, then turn off the int!
	andi	TTEMP,$BF	;
	out	GIMSK,TTEMP	;

INT0_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti
;
;***************************************************************
;
;
EXT_INT1:
	in	TTEMP2,SREG	;Saves the status register

INT1_OFF:
	in	TTEMP,GIMSK	;Get the int enables
	andi	TTEMP,$7F	;Mask off INT1
	out	GIMSK,TTEMP	;Send it
	ldi	TTEMP,$80	;
	out	GIFR,TTEMP	;Manual clear
	
INT1_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;
TIM1_CAPT:			;
	in	TTEMP2,SREG	;Saves the status register

TIM1_OFF:
	in	TTEMP,TIMSK	;Shut off this int
	andi	TTEMP,$F7	;
	out	TIMSK,TTEMP	;

TIM1_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti		;
;
;***************************************************************
;
;
;
TIM1_COMPA:			;
	in	TTEMP2,SREG	;Saves the status register

TC1A_OFF:
	in	TTEMP,TIMSK	;
	andi	TTEMP,$BF	;
	out	TIMSK,TTEMP	;

TC1A_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti			;
;	
;***************************************************************
;
;
TIM1_COMPB:		
	in	TTEMP2,SREG	;Saves the status register

TC1B_OFF:
	in	TTEMP,TIMSK	;
	andi	TTEMP,$DF	;
	out	TIMSK,TTEMP	;

TC1B_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Timer 1 gives us servo output timings from 1mS to 2mS, and allows
;the CPU to go do other things, rather than sitting there spinning 
;NOPS. It's a 16 bit counter, so I start it with a base value of 1mS,
;plus the servo width control byte (SERVO_X) We then have 1-2mS pulses,
;with full 8 bit resolution (4uS)
;
;Each servo out is activated sequentially in the main code. The activation
;code loads a value into Timer1, and sets the prescaler to /8, and sets the
;desired servo bit high. This int occurs when the servo's time expires, and
;we just shut off the output, and stop the timer. (Fast eh? :)
;
TIM1_OVF:
	in	TTEMP2,SREG	;Saves the status register

	;**************************************************
	;Support for the R/C servo driver in SERVO.ASM
	;**************************************************

	;No matter who was running, shut off all servo outputs
	ldi	TTEMP,$00	;
	out	PORTC,TTEMP	;

	in	TTEMP,TCCR1B	;Get the timer control register
	andi	TTEMP,$F8	;Set the prescaler to "off"
	out	TCCR1B,TTEMP	;Make it so.

	;This test could instead mask and look for a particular
	;bit to reduce the number of active servos.

	lds	TTEMP,Servo_Control	;

	;To make the servos run from 1 to 8, enable this
	lsl	TTEMP			;Set next servo to run
	brcc	Servo_TB		;If no carry then done, else
	ldi	TTEMP,$01		;start over on servo 1(0)

	;To make the servos run from 8 to 1, enable this
	;lsr	TTEMP			;Set next servo to run
	;brcc	Servo_TB		;If no carry then done, else
	;ldi	TTEMP,$80		;start over on servo 1(0)

	;If there was a ROL without carry, then I wouldn't have needed
	;to test for carry and reload like that. can't use LSL

Servo_TB:
	sts	Servo_Control,TTEMP	;Set the next active servo bit on.

TIM1O_EXIT:
	out	SREG,TTEMP2		;restore the status register
	reti				;
;
;***************************************************************
;
;	Main 1mS Opsys tick
;
TIM0_OVF:

	in	TTEMP3,SREG	;Saves the status register

	;**************************************************
	;Reload timer 0 now to preserve accuracy
	;**************************************************
	ldi	TTEMP,T0DIV	;Reload early, avoid the latency
	out	TCNT0,TTEMP	;

	;**************************************************
	;LCD Display scroll timer support
	;**************************************************
T0_LCD	:
	lds	TTEMP,LCD_Timer	;
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_LCD_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	LCD_Timer,TTEMP	;and store
T0_LCD_Done:	;Dummy label

	;**************************************************
	;VFD Display scroll timer support
	;**************************************************
T0_VFD	:
	lds	TTEMP,VFD_Timer	;
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_VFD_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	VFD_Timer,TTEMP	;and store
T0_VFD_Done:	;Dummy label

	;**************************************************
	;Morse code support
	;**************************************************
T0_Morse:

	;First, if we aren't active, then just ignore
	lds	TTEMP,BEEPFLAG	;Status check
	rol	TTEMP		;Active?
	brcc	T0_M_D		;If not, do something else.

	andi	TTEMP,$40	;Is the end flag set?
	breq	T0_Beep_State	;If not, go look at beeps

	;We get to here if beepflag is xx1xxxxx, meaning we are
	;doing the trailing silence

T0_End_Beep:
	lds	TTEMP,BEEPTIME	;Counting down
	dec	TTEMP		;One less
	sts	BEEPTIME,TTEMP	;Save it
	and	TTEMP,TTEMP	;Is it zero?
	brne	T0_M_D		;If not, just fly on
	
	lds	TTEMP,BEEPTIME2	;More to go?
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_End_Beep2	;If so, then we're done
	dec	TTEMP		;Now it's one less
	sts	BEEPTIME2,TTEMP	;Save it
	ldi	TTEMP,$FF	;and reload beeptime for another round
	sts	BEEPTIME,TTEMP	;
	rjmp	T0_M_D		;And bail, no sense checking further
				;until this one is empty

T0_End_Beep_A:
	lds	TTEMP,Dots_2_Go	;Any more "dots" to send?
	and	TTEMP,TTEMP	;
	breq	T0_End_Beep2	;Nope, done

	dec	TTEMP		;One less dot to send
	sts	Dots_2_Go,TTEMP	;Save that
	lds	TTEMP,M_SPEED	;Get a dot-time
	sts	BEEPTIME,TTEMP	;Reload the counter
	rjmp	T0_M_D		;Done

T0_End_Beep2:
	clr	TTEMP		;Flag it inactive
	sts	BEEPFLAG,TTEMP	;
	cbi	Morse_Port,Morse_Bit;Make sure of the exit state
	rjmp	T0_M_D		;

	;If we're in the middle of a beep or silent, then
	;dec and carry on. If we hit the end, then it's time
	;to change the state
T0_Beep_State:
	lds	TTEMP,BEEPTIME	;
	dec	TTEMP		;
	sts	BEEPTIME,TTEMP	;
	brne	T0_M_D		;If non-zero, then we're done

	;If zero:
	lds	TTEMP,Dots_2_Go	;Any more "dots" to send?
	and	TTEMP,TTEMP	;
	breq	T0_M_Change_State ;Nope, done

	dec	TTEMP		;One less dot to send
	sts	Dots_2_Go,TTEMP	;Save that
	lds	TTEMP,M_SPEED	;Get a dot-time
	sts	BEEPTIME,TTEMP	;Reload the counter
	rjmp	T0_M_D ;Done

	;At the beginning, the count is zero, and the state
	;is beep. 
	;Each time we come through, we fetch another bit-pair
	;which indicates dot (short beep), dash (long beep)
	;or end. 
	;For each bit pair, we do a beep of either one or three
	;units long, and we do a one unit silence.
	;If the state is beep, and the fetched value is end,
	;then we set bit 7 to 0, and exit

T0_M_D: rjmp	T0_M_Done	;Trick for long rel jmp

T0_M_Change_State:
	lds	TTEMP,BEEPFLAG	;
	mov	TTEMP2,TTEMP	;A copy
	andi	TTEMP,$40	;
	breq	T0_M_Beep	;

T0_M_Sil:
	mov	TTEMP,TTEMP2	;fresh copy
	andi	TTEMP,$BF	;Reset silent bit
	ori	TTEMP,$80	;Set active bit
	sts	BEEPFLAG,TTEMP	;
	cbi	Morse_Port,Morse_Bit;Output silent
	lds	TTEMP,M_SPEED	;Inter-symbol silence is
	sts	BEEPTIME,TTEMP	;always one unit
	ldi	TTEMP,0		;
	sts	Dots_2_Go,TTEMP	;
	rjmp	T0_M_Done	;We're done, till the silence
				;is over
T0_M_Beep:
	lds	TTEMP,BEEPFLAG	;
	inc	TTEMP		;Next State
	andi	TTEMP,$07	;
	ori	TTEMP,$C0	;Set flag bit and active bit
	sts	BEEPFLAG,TTEMP	;
	sbi	Morse_Port,Morse_Bit;output beep

T0_M_Lookup:
	;TTEMP has 0-7 now
	;and I need to know what di-bit in MBITS and MBITS2
	;correspond
	;
	mov	TTEMP2,TTEMP	;copy
	andi	TTEMP2,$04	;high bit only
	breq	T0_Get_MB	;

T0_Get_MB2:
	lds	TTEMP2,MBITS2	;
	rjmp	T0_M_Bits	;

T0_Get_MB:
	lds	TTEMP2,MBITS1	;

;A long, but fast strip of the appropriate bits.
	
T0_M_Bits:
	andi	TTEMP,$03	;Pick the right bit-pair
	breq	T0_M_B0		;0-3 = AA BB CC DD
	cpi	TTEMP,$01	;One
	breq	T0_M_B1		;
	cpi	TTEMP,$02	;Two
	breq	T0_M_B2		;
	rjmp	T0_M_B3		;Three

T0_M_B3:
	andi	TTEMP2,$03	;Mask
	breq	T0_M_Dit	;Handle 00 (*)
	cpi	TTEMP2,$01	;
	breq	T0_M_Dah	;Handle 01 (-)
	rjmp	T0_M_End	;Handle END

T0_M_B2:
	andi	TTEMP2,$0C	;Mask
	breq	T0_M_Dit	;Handle 00 (*)
	cpi	TTEMP2,$04	;
	breq	T0_M_Dah	;Handle 01 (-)
	rjmp	T0_M_End	;Handle END

T0_M_B1:
	andi	TTEMP2,$30	;
	breq	T0_M_Dit	;Handle 00 (*)
	cpi	TTEMP2,$10	;
	breq	T0_M_Dah	;Handle 01 (-)
	rjmp	T0_M_End	;Handle END

T0_M_B0:
	andi	TTEMP2,$C0	;
	breq	T0_M_Dit	;Handle 00 (*)
	cpi	TTEMP2,$40	;
	breq	T0_M_Dah	;Handle 01 (-)
	rjmp	T0_M_End	;Handle END

T0_M_Dit:
	lds	TTEMP,M_SPEED	;get the time reload
	sts	BEEPTIME,TTEMP	;save it
	ldi	TTEMP,0		;
	sts	BEEPTIME2,TTEMP	;
	sts	Dots_2_Go,TTEMP	;
	rjmp	T0_M_Done	;

T0_M_Dah:
	lds	TTEMP,M_SPEED	;get the time reload
	sts	BEEPTIME,TTEMP	;save it
	ldi	TTEMP,0		;
	sts	BEEPTIME2,TTEMP	;
	ldi	TTEMP,2		;
	sts	Dots_2_Go,TTEMP	;
	rjmp	T0_M_Done	;

T0_M_End:
	lds	TTEMP,BEEPFLAG	;
	andi	TTEMP,$BF	;Reset the beeping flag
	ori	TTEMP,$20	;Flag for three units of silence
	sts	BEEPFLAG,TTEMP	;

	lds	TTEMP,M_SPEED	;get the time reload
	sts	BEEPTIME,TTEMP	;
	ldi	TTEMP,0		;
	sts	BEEPTIME2,TTEMP	;

	ldi	TTEMP,1		;
	sts	Dots_2_Go,TTEMP	;
	cbi	Morse_Port,Morse_Bit;Output a silence
	rjmp	T0_M_Done	;	

T0_M_Done:	;Dummy label, fallthrough

	;**************************************************
	;Support for the stepper motor handler
	;**************************************************

T0_Step:lds	TTEMP,Step_Time	;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_Dog		;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Step_Time,TTEMP	;and store

	;**************************************************
	;Support for the timed watchdog handler
	;**************************************************

T0_Dog:	lds	TTEMP,Dog_Food	;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_Char		;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Dog_Food,TTEMP	;and store

	;**************************************************
	;Support for the timed charachter wait
	;**************************************************

T0_Char:lds	TTEMP,Char_Time	;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_Char_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Char_Time,TTEMP	;and store

T0_Char_Done: ;Dummy Label
	;**************************************************
	;Support for the R/C servo driver in SERVO.ASM
	;**************************************************

T0_RC:	lds	TTEMP,Frame_Delay;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_RC_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Frame_Delay,TTEMP;and store

T0_RC_Done: ;Dummy Label

	;**************************************************
	;Support for the inter-char delay feature in SERIAL.ASM
	;**************************************************
T0_SER:	lds	TTEMP,Char_Delay	;
	and	TTEMP,TTEMP		;
	breq	T0_Ser_Check		;If already zero, nevermind

	dec	TTEMP			;else dec, and if NOW zero
	sts	Char_Delay,TTEMP	;
	brne	T0_Ser_Done		;If still not zero, just exit

	;Else check for stuff to send
T0_Ser_Check:
	push	YL			;
	push	YH			;

	ld	TTEMP,Y			;Get the size
	and	TTEMP,TTEMP		;Is it zero?
	breq	T0_Ser_Empty		;If so, exit
	sbi	UCR,UDRE		;else turn on the UDRE int

T0_Ser_Empty:
	pop	YH			;
	pop	YL			;

T0_Ser_Done:  ;Dummy Label
	

	;**************************************************
	;All done, seeya!
	;**************************************************
TIM0_EXIT:
	out	SREG,TTEMP3	;restore the status register
	reti			;Bye!
;
;***************************************************************
;
;SPI is unused at this point
;
SPI_Handler:			;
	in	TTEMP2,SREG	;Saves the status register

SPI_OFF:
	ldi	TTEMP,$3F	;Disable int, disable SPI
	out	SPCR,TTEMP	;

SPI_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Take a char and put it in the Serial input buffer. 
;
UART_RXC:			;
	in	TTEMP2,SREG	;Saves the status register
	push 	TEMP		;Have to use TEMP because Store_Serin uses temp

	;**************************************************
	;Loading incoming UART data into the serial input buffer
	;**************************************************
	in	TEMP,UDR	;Get the char
	rcall	Store_Serin	;Store it in the serial in buffer (USES TEMP)

UART_RXC_Exit:
	pop	TEMP		;
	out	sreg,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Ready to transmit the next byte
;
UART_DRE:			;
	in	TTEMP2,SREG	;Saves the status register
	push	YL
	push	YH


	ldi	YL,low(SERIAL_OUT_BUF)	;
	ldi	YH,high(SERIAL_OUT_BUF)	;
	ld	TTEMP,Y+		;Get the current size
	and	TTEMP,TTEMP		;
	brne	UART_DRE_TX	;If non-zero, then let it continue
	
	cbi	UCR,UDRE	;
	rjmp	UART_DRE_EXIT	;

UART_DRE_TX:

	lds	TTEMP,(Serial_OUT_BUF+2);Load a byte to temp from SERIAL_OUT_BUF

	;
	; Time------------>
	;  S L X X X X X M S S
	;  T S X X X X X S P T 
	;  A B X X X X X B C P
	;1 0 0 0 0 0 0 0 0 0 1  (Line state)
	;
	;Comment both of the following out for 8N1
	andi	TTEMP,$7F		;7 bit Space parity
	;ori	TTEMP,$80		;7 bit Mark  parity
	out	UDR,TTEMP		;Load uart from TEMP

	ldi	TTEMP,SER_DLY		;Stupid inter-char delay
	sts	Char_Delay,TTEMP	;

	;disable Uart_DRE_int.
	cbi	UCR,UDRE		;

	ldi	YL,low(SERIAL_OUT_BUF)	;
	ldi	YH,high(SERIAL_OUT_BUF) ;
	push	TEMP			;
	push	TEMP2			;
	rcall	Kill_Head		;Kill the sent byte
	pop	TEMP2			;
	pop	TEMP			;


UART_DRE_EXIT:
	pop	YH		;
	pop	YL		;
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Uart TXC is unused at this point
;
UART_TXC:
	in	TTEMP2,SREG	;Saves the status register

UTXC_OFF:
	in	TTEMP,UCR	;
	andi	TTEMP,$BF	;Disable the TXC int.
	out	UCR,TTEMP	;

UTXC_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti			;
;
;***************************************************************
;
;Analog comparator is unused at this point.
;
ANA_COMP:
	in	TTEMP2,SREG	;Saves the status register

ANAC_OFF:
	ldi	TTEMP,$83	;Shut off power to the comparator
	out	ACSR,TTEMP	;and the comparator ints

ANAC_EXIT:
	out	SREG,TTEMP2	;restore the status register
	reti
;
;***************************************************************
;