
;***************************************************************************
; 
; File Name		:'SERIAL.asm"
; Title			:Serial I/O
; Date			:98.07.21
; Version		:0.01
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Target MCU		:AT90S8515
;
; DESCRIPTION
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.07.29  dvh	Creation
;	0,02	98.08.29  dvh	Added break and "wait for char"
;	0.03	98.09.01  dvh	Working on making it more interrupt driven
;	0.04	99.01.07  dvh	Simpler break width generation
;	0.05	99.09.11  dvh	Moved the init in here.
;
;***************************************************************************
;I'm not really satisfied with this yet, I'm not having a lot of luck modeling
;a fully interrupt-driven comm scheme along with things like the "wait for char"
;feature. Also, there's a conflict if the output buffer fills up. There is just
;no good way to handle that as far as I can see. I always end up with a construct
;that loops forever and dosen't return to the main system. 
;
;Your input is most welcome here :)
;***************************************************************************************
;
Init_Serial:
	;98.07.27 Added this to initialize the HS status before
	;enabling int driven comms. Note that we do not SEND xon 
	;here, just establishing a known state.  
	;This MUST be done prior to enabling int driven comms
	;
	ldi	TEMP,$FF	;Set serial input state to XON
	sts	HS_FLAG,TEMP	;

	ldi	TEMP,$00	;Set the inter-char timer to zero
	sts	Char_Delay,TEMP	;

	rcall	Set_Baud_9600	;Or something else if you prefer (SERIAL.ASM)

	cbi	UCR,CHR9	;No 9 bit funny stuff
	sbi	UCR,RXCIE	;Enable the RX complete interrupt
	sbi	UCR,TXCIE	;Enable the TX complete interrupt
	sbi	UCR,UDRIE	;Enable the Uart Data Register Empty int
	sbi	UCR,RXEN	;Enable Uart Receive (takes pin D0)
	sbi	UCR,TXEN	;Enable Uart Transmit (takes pin D1)

	;in	TEMP,UCR	;Enable TX and RX ints
	;ori	TEMP,$C0	;
	;out	UCR,TEMP	;
	ret
;
;***************************************************************************************
;
Ser_TEST1:
	ldi	ZL,low (Signon_Message*2) ;Point at a signon message (in TABLES.ASM)
	ldi	ZH,high (Signon_Message*2) ;
	ldi	YL,low(Serial_Out_Buf)	;
	ldi	YH,high(Serial_Out_Buf)	;
	rcall	Test_Space		;Returns zero if the string won't fit
					;in the target buffer.
	breq	Ser_Test_Done		;
	rcall	String_out		;String to buf, buf to LCD
Ser_Test_Done:
	ret
;
;********************************************************************
;
;Tom Hubrin's Method:
;From the data sheet:BAUD = XTAL/(16*(UBRR+1))
;Rearranging you get:UBRR = XTAL/(16*BAUD) - 1
;
;This would work except that any fraction will be truncated. 
;For least error we want to round off the result. 
;So we add 1/2 as follows: UBRR = XTAL/(16*BAUD) - 1 + 0.5
;Combining the -1 and the +0.5 we can rewrite this as follows:
;
;UBRR = XTAL/(16*BAUD) - 0.5
;
;The problem is that we cannot work with fractions in 
;integer math. But we can rearrange it so that the end 
;result is the same without using any decimal points.
;
;UBRR = XTAL/(16*BAUD) - 0.5 * (16*BAUD)/(16*BAUD)
;UBRR = XTAL/(16*BAUD) - (8*BAUD)/(16*BAUD)
;So: UBRR = ( XTAL - 8*BAUD )/( 16*BAUD )
;
;Note that we are NOT doing any of this in the machine.
;The assembler simply uses the most recently declared value
;of BaudRate to evaluate the expression. The result is
;really "ldi TEMP,N" where N is the right divisor for the
;desired baud rate.

Set_Baud_2400:
	.set BaudRate=2400
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall SET_BAUD
	ret

Set_Baud_4800:
	.set BaudRate=4800
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_9600:
	.set BaudRate=9600
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_14400:
	.set BaudRate=14400
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_19200:
	.set BaudRate=19200
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_28800:
	.set BaudRate=28800
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_57600:
	.set BaudRate=57600
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud_115200:
	.set BaudRate=115200
	ldi TEMP, ( XTAL - 8*BaudRate ) / ( 16*BaudRate )
	rcall	SET_BAUD	;
	ret

Set_Baud:

	mov	LOOP,TEMP	;Save it for a moment
	rcall	WT4TXMT		;Wait till all prev sent data is gone
	out	UBRR,LOOP	;Set the baud rate
	ret			;
;
;************************************************************
;
;This one waits for TX buffer empty
;
WT4TXMT:
	rcall	Timed_Smack		;Reset the watchdog if needed
	ldi	YL,low(SERIAL_OUT_BUF)	;
	ldi	YH,high(SERIAL_OUT_BUF)	;
	rcall	Check_It		;
	breq	WT4TXMT			;
	ret
;
;This one waits till the current char is done transmitting
;
WT4XCHR:
	rcall	Timed_Smack	;Reset watchdog if needed
	in	TEMP,USR	;
	andi	TEMP,$40	;Get the TX Complete bit
	breq	WT4XCHR		;
	ret
;
;************************************************************
;
;Set output handshaking.
;Toggled based on current status
;Jam XON or XOFF into output buffer
;
HS_XOFF:
	lds	TEMP,HS_FLAG	;
	and	TEMP,TEMP	;
	breq	Ser_HS_Done	;

	ldi	TEMP,XOFF	;Jam XOFF into serial out buffer
	ldi	YH,high(Serial_Out_Buf)	;Get the buffer head
	ldi	YL,low(Serial_Out_Buf)	;
	rcall	Store_Buffer	;
	
	;Enable transmit (if not already)

	in	TEMP,UCR	;
	andi	TEMP,$7F	;
	out	UCR,TEMP	;Make sure no more data
	in	TEMP,UCR	;
	ori	TEMP,$20	;Turn on UDRIE
	out	UCR,TEMP	;in case it's off, so xoff and xon get sent
	ldi	TEMP,$00	;
	sts	HS_FLAG,TEMP	;
	rjmp 	SER_HS_Done	;

;
;************************************************************
;
HS_XON:
	lds	TEMP,HS_FLAG	;Get current status
	and 	TEMP,TEMP	;
	brne	Ser_HS_Done	;If non-zero, then it's already cleared

	ldi	TEMP,XON	;Jam XON into serial out buffer
	ldi	YH,high(Serial_Out_Buf)	;Get the buffer head
	ldi	YL,low(Serial_Out_Buf)	;
	rcall	Store_Buffer	;

	in	TEMP,UCR	;Enable transmit if not already
	ori	TEMP,$80	;
	out 	UCR,TEMP	;
	in	TEMP,UCR	;
	ori	TEMP,$20	;Turn on UDRIE
	out	UCR,TEMP	;in case it's off, so xoff and xon get sent
	ldi	TEMP,$FF	;
	sts	HS_FLAG,TEMP	;

Ser_HS_Done:
	ret
;
;************************************************************
;
SEND_BREAK:

	;Assumption, that you want anything placed in the serial buffer
	;to be sent before the break. 
	rcall	WT4TXMT		;Wait for serial buffer empty
	rcall	WT4XCHR		;Wait for the last char to get out

	;Shift the baud rate slightly lower.
	in	TEMP,UBRR	;Get the current baud rate (51 @ 9600)
	push	TEMP		;Save it
	;mov	LOOP,TEMP	;Make a copy

	;clc			;
	;lsr	TEMP		;divide it by 8
	;lsr	TEMP		;
	;lsr	TEMP		;
	;add	LOOP,TEMP	;Add that to the old value

	;out	UBRR,LOOP	;Set the new baudrate, two "bits" slower

	ldi	TEMP,255	;Much simpler, works for most baud rates.
	out	UBRR,TEMP	;

	ldi	TEMP,NUL	;Load a null, 8 bits of zero
	out	UDR,TEMP	;Start Breaking now.	

	rcall	WT4XCHR		;Wait till the char is sent

	pop	TEMP		;Get the old baud rate
	out	UBRR,TEMP	;Restore the original baud rate

	ret
;
;************************************************************
;
;Input: Char to get in temp, Delay in TEMP2
;Output: TEMP2=0 for ok, or FF for timeout or wrong char
;
WT4CHAR: ;This starts the Serial_Wait engine to look for an incoming char.
	
	sts	Char_Time,TEMP2	;System will decrement on 1mS system ticks
	sts	Watch_Char,TEMP	;save temp for compare
	;Turn off inbound serial ints
	ret			;

;Char_Time = 0, Not waiting, or timed out.
;Char_Time <> 0, actively waiting.
;Watch_Char = byte value waiting on.
;


Serial_Wait:
	lds	TEMP,Char_Time	;Are we waiting on a char (0=no)
	and	TEMP,TEMP	;Is Char_Time=0?
	brne	WT4CHAR_ACTIVE	;If not, then we're waiting
	ret			;Else return

	;If we're waiting, then see if it's happened

WT4CHAR_ACTIVE:
	in	TEMP,USR	;Did we get a char yet?
	andi	TEMP,$80	;Check bit 7
	brne	WT4CHAR_GOTIT	;
	ret			;Nope, just keep waiting

WT4CHAR_GOTIT:

	in	TEMP2,UDR	;Get the Char
	andi	TEMP2,$7F	;Mask off parity(?)
	lds	TEMP,Watch_Char	;What were we looking for?
	cp	TEMP,TEMP2	;Is this it?
	breq	WT4CHAR_SUCCESS	;

WT4CHAR_SUCCESS:
	ret


;
;***************************************************************************************
;
